
#include <Wire.h>             // Подключить библиотеку Wire library (требуется для устройств I2C)
#include <Adafruit_GFX.h>     // Adafruit графическая библиотека
#include <Adafruit_ST7789.h>  // Adafruit библиотека для ST7789
#include "RTClib.h"           // Adafruit RTC библиотека для работы с часами

// ST7789 TFT Соединение с модулем
#define TFT_CS    10          
#define TFT_DC     9          
#define TFT_RST    8          


Adafruit_ST7789 tft = Adafruit_ST7789(TFT_CS, TFT_DC, TFT_RST);

// Инициализация RTC библиотеки
RTC_DS3231 rtc;
DateTime   now;

#define button1  7            // Кнопка запуска настройки
#define button2  6            // Кнопка настройки часов и даты

void setup(void)
{
  tft.init(240, 240, SPI_MODE2);    //  ST7789 дисплей 240x240 px
  
  tft.setRotation(2);           // Поворот экрана  
  tft.fillScreen(ST77XX_BLACK); // Цвет заливки

  tft.fillRect(0, 32, tft.width(), 2, ST77XX_BLUE);
  tft.fillRect(0, 111, tft.width(), 2, ST77XX_BLUE);
  tft.fillRect(0, 190, tft.width(), 2, ST77XX_BLUE);

  tft.setTextWrap(false);                         // отключить опцию переноса текста
  tft.setTextColor(ST77XX_WHITE, ST77XX_BLACK);   // установить цвет текста и черный фона
  tft.setTextSize(2);                             // Размер шрифта = 2
  tft.setCursor(90, 0);                            // переместить курсор в положение (5, 0)
  tft.print(utf8rus("МОСКВА"));
  tft.setCursor(55, 16);                          // переместить курсор в положение (55, 16)
  tft.print(utf8rus("ТОЧНОЕ ВРЕМЯ"));
  tft.setCursor(50, 194);             
  tft.print(utf8rus("ТЕМПЕРАТУРА:"));
  tft.setTextSize(4);                             // Размер шрифта = 4
  tft.setTextColor(ST77XX_MAGENTA, ST77XX_BLACK);
  tft.setCursor(55, 120);
  tft.print(utf8rus("ВРЕМЯ:"));

  pinMode(button1, INPUT_PULLUP);
  pinMode(button2, INPUT_PULLUP);

  rtc.begin();                                    // Запуск RTC библиотеки
}

// Защита от дребезга
bool debounce ()
{
  byte count = 0;
  for(byte i = 0; i < 5; i++)
  {
    if ( !digitalRead(button1) )
      count++;
    delay(10);
  }

  if(count > 2)  return 1;
  else           return 0;
}

void RTC_display()
{
  char _buffer[11];
  
//Выводим дни недели  
//  char dow_matrix[7][10] = {"SUNDAY", "MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY"};
  char dow_matrix[7][10] = {"Вс.", "Пн.", "Вт.", "Ср.", "Чт.", "Пт.", "Сб."};
  byte x_pos[7] = {10, 10, 10, 10, 10, 10, 10};
  static byte previous_dow = 8;

 if( previous_dow != now.dayOfTheWeek() )
  {
    previous_dow = now.dayOfTheWeek();
    tft.fillRect(14, 41, 216, 28, ST77XX_BLACK);     // Рисуем прямоугольник и заливаем чёрным цветом
    tft.setCursor(x_pos[previous_dow], 41);
    tft.setTextColor(ST77XX_CYAN, ST77XX_BLACK);     // День недели. Голубой на чёрном
    tft.print(utf8rus(dow_matrix[now.dayOfTheWeek()]));
  }

  // Форматируем и выводим дату ДЕНЬ-МЕСЯЦ-ГОД
  sprintf( _buffer, "%02u-%02u-%04u", now.day()%100, now.month()%100, now.year() );
  tft.setCursor(1, 76);
  tft.setTextColor(ST77XX_YELLOW, ST77XX_BLACK);  // Жёлтый на Чёрном
  tft.print(_buffer);
  // Форматируем и выводим время ЧАСЫ-МИНУТЫ-СЕКУНДЫ
  sprintf( _buffer, "%02u:%02u:%02u", now.hour()%100, now.minute()%100, now.second()%100 );
  tft.setCursor(26, 155);
  tft.setTextColor(ST77XX_GREEN, ST77XX_BLACK);  // Зелёный на Чёрном
  tft.print(_buffer);
}

// Установка даты и времени
byte edit(byte parameter)
{
  static byte i = 0, y_pos,
              x_pos[5] = {1, 73, 193, 26, 98};
  char text[3];
  sprintf(text,"%02u", parameter);

  if(i < 3) {
    tft.setTextColor(ST77XX_YELLOW, ST77XX_BLACK);  
    y_pos = 76;
  }
  else {
    tft.setTextColor(ST77XX_GREEN, ST77XX_BLACK);  
    y_pos = 155;
  }

  while( debounce() );  

  while(true) {
    while( !digitalRead(button2) ) {  // Нажата кнопка 2 
      parameter++;
      if(i == 0 && parameter > 31)    // Если день > 31 ==> день = 1
        parameter = 1;
      if(i == 1 && parameter > 12)    // Если месяц > 12 ==> месяц = 1
        parameter = 1;
      if(i == 2 && parameter > 99)    // Если год > 99 ==> год = 0
        parameter = 0;
      if(i == 3 && parameter > 23)    // Если часы > 23 ==> часы = 0
        parameter = 0;
      if(i == 4 && parameter > 59)    // Если минуты > 59 ==> минуты = 0
        parameter = 0;

      sprintf(text,"%02u", parameter);
      tft.setCursor(x_pos[i], y_pos);
      tft.print(text);
      delay(200);      
    }

    tft.fillRect(x_pos[i], y_pos, 44, 28, ST77XX_BLACK);
    unsigned long previous_m = millis();
    while( (millis() - previous_m < 250) && digitalRead(button1) && digitalRead(button2)) ;
    tft.setCursor(x_pos[i], y_pos);
    tft.print(text);
    previous_m = millis();
    while( (millis() - previous_m < 250) && digitalRead(button1) && digitalRead(button2)) ;

    if(!digitalRead(button1))
    {                                 // Если нажата кнопка 1
      i = (i + 1) % 5;                // Увеличиваем значение
      return parameter;               // Возвращаем полученное значение
    }
  }
}


void loop()
{
  if( !digitalRead(button1) )  // Если нажата кнопка 1
  if( debounce() )            
  {
    while( debounce() );  

    byte day    = edit( now.day() );          // Редактируем дату
    byte month  = edit( now.month() );        // Редактируем месяц
    byte year   = edit( now.year() - 2000 );  // Редактируем год
    byte hour   = edit( now.hour() );         // Редактируем час
    byte minute = edit( now.minute() );       // Редактируем минуты

    // write time & date data to the RTC chip
    rtc.adjust(DateTime(2000 + year, month, day, hour, minute, 0));

    while(debounce());  
  }

  now = rtc.now();  // считывает текущее время и дату с чипа RTC

  RTC_display();   
  
  // Считываем температуру с модуля часов
  Wire.beginTransmission(0x68); 
  Wire.write(0x11);             
  Wire.endTransmission(false);  
  Wire.requestFrom(0x68, 2);    
  byte t_msb = Wire.read();     
  byte t_lsb = Wire.read();     

  // Выводим температуру
  char _buffer[6];
  uint16_t chip_temp = (uint16_t)t_msb << 2 | t_lsb >> 6;

  if(t_msb & 0x80) {
    chip_temp |= 0xFC00;
    sprintf(_buffer, "-%02u.%02u", abs((int)chip_temp * 25) / 100, abs((int)chip_temp * 25) % 100);
  }
  else
    sprintf(_buffer, " %02u.%02u", (chip_temp * 25) / 100, (chip_temp * 25) % 100);

  tft.setCursor(14, 212);
  tft.setTextColor(ST77XX_RED, ST77XX_BLACK);  // Печатаем температуру красным
  tft.print(_buffer);
  tft.drawCircle(173, 218, 4, ST77XX_RED);     // Выводим знак градуса ( ° )
  tft.drawCircle(173, 218, 5, ST77XX_RED);
  tft.setCursor(184, 212);
  tft.print("C");
  
  delay(100);    
}



/* Вывод русского шрифта Перевод из  UTF-8 в Windows-1251 */

String utf8rus(String source)
{
  int i,k;
  String target;
  unsigned char n;
  char m[2] = { '0', '\0' };

  k = source.length(); i = 0;

  while (i < k) {
    n = source[i]; i++;

    if (n >= 0xC0) {
      switch (n) {
        case 0xD0: {
          n = source[i]; i++;
          if (n == 0x81) { n = 0xA8; break; }
          if (n >= 0x90 && n <= 0xBF) n = n + 0x30;
          break;
        }
        case 0xD1: {
          n = source[i]; i++;
          if (n == 0x91) { n = 0xB8; break; }
          if (n >= 0x80 && n <= 0x8F) n = n + 0x70;
          break;
        }
      }
    }
    m[0] = n; target = target + String(m);
  }
return target;
}
